@@ -21,6 +21,11 @@ class SeriesJobConfigModel(dict):
             raise Exception(
                 "Invalid series job config, module property must be a string")
 
+        if len(module.split("@")) != 2:
+            raise Exception(
+                "Invalid series job config, module property must have format "
+                "<module_name>@<module_version>")
+
         if job_type not in JOB_TYPES:
             raise Exception(
                 "Invalid series job config, unknown job_type")
@@ -45,7 +50,8 @@ class SeriesJobConfigModel(dict):
                 "Invalid series job config, "
                 "module_params property must be a dict")
 
-        if max_n_points is not None and not isinstance(max_n_points, int):
+        if max_n_points is not None and not isinstance(
+                max_n_points, int):
             raise Exception(
                 "Invalid series job config, "
                 "max_n_points property must be an integer")
@@ -57,6 +63,10 @@ class SeriesJobConfigModel(dict):
 
         if config_name is None:
             config_name = str(uuid.uuid4())
+        elif " " in config_name:
+            raise Exception(
+                "Invalid series job config, "
+                "config_name must not contains any spaces")
 
         super(SeriesJobConfigModel, self).__init__({
             "activated": activated,
@@ -76,7 +86,11 @@ class SeriesJobConfigModel(dict):
 
     @property
     def module(self):
-        return self.get("module")
+        return self.get("module").split("@")[0]
+
+    @property
+    def module_version(self):
+        return self.get("module").split("@")[1]
 
     @property
     def job_type(self):
